from dockit.schema.schema import create_document, create_schema

from django.utils import unittest

from common import SimpleSchema

class SchemaTestCase(unittest.TestCase):
    def test_to_primitive(self):
        obj = SimpleSchema(_python_data={'charfield':'charmander'})
        prim_data = obj.to_primitive(obj)
        self.assertEqual(prim_data, {'charfield':'charmander'})
    
    def test_to_python(self):
        obj = SimpleSchema(_primitive_data={'charfield':'charmander'})
        py_obj = obj.to_python({'charfield':'charmander'})
        self.assertEqual(obj._primitive_data, py_obj._primitive_data)
    
    def test_traverse(self):
        obj = SimpleSchema(charfield='charmander')
        self.assertEqual(obj.dot_notation('charfield'), 'charmander')




from dockit.schema import fields

from django.utils import unittest
from django.contrib.contenttypes.models import ContentType

import datetime
from decimal import Decimal

from common import SimpleSchema, SimpleDocument

class BaseFieldTestCase(unittest.TestCase):
    EXAMPLE_VALUES = []
    EXAMPLE_PRIMITIVE_VALUES = []
    NULL_VALUE = None
    field_class = fields.BaseField
    
    def get_example_values(self):
        return self.EXAMPLE_VALUES
    
    def get_example_primitive_values(self):
        return self.EXAMPLE_PRIMITIVE_VALUES
    
    def get_field_kwargs(self):
        return {}
    
    def get_field(self, **kwargs):
        params = self.get_field_kwargs()
        params.update(kwargs)
        return self.field_class(**params)
    
    def test_handles_null_value(self):
        field = self.get_field(null=True)
        val = field.to_python(None)
        self.assertEqual(val, self.NULL_VALUE)
    
    def test_to_python_to_primitive(self):
        field = self.get_field()
        
        for val in self.get_example_values():
            primitive = field.to_primitive(val)
            py_val = field.to_python(primitive)
            self.assertEqual(py_val, val)
        
        for val in self.get_example_primitive_values():
            py_val = field.to_python(val)
            primitive = field.to_primitive(py_val)
            self.assertEqual(val, primitive)
        
    def test_form_field(self):
        field = self.get_field()
        field.formfield()
    
    def test_get_choices(self):
        field = self.get_field()
        field.get_choices()
    
    def test_json_serializable(self):
        field = self.get_field()
        for val in self.get_example_values():
            primitive = field.to_primitive(val)
            #TODO attempt to serialize
    
    def test_traverse_dotpath(self):
        pass #TODO raise skipped test

class CharFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = ["abc", u"def"] #TODO good unicode values
    field_class = fields.CharField

class IntegerFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = [1, 3243455]
    field_class = fields.IntegerField

class BigIntegerFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = [1, 3243455, 2**32+1]
    field_class = fields.BigIntegerField

class BooleanFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = [True, False]
    field_class = fields.BooleanField

class DateFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = [datetime.date(2001,1,1)]
    field_class = fields.DateField

class DateTimeFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = [datetime.datetime(2001,1,1)]
    field_class = fields.DateTimeField

class DecimalFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = [1, 2, Decimal('1.5')]
    field_class = fields.DecimalField

class EmailFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = ['z@z.com']
    field_class = fields.EmailField

class IPAddressFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = ['127.0.0.1']
    field_class = fields.IPAddressField

class SlugFieldTestCase(BaseFieldTestCase):
    EXAMPLE_VALUES = ['slug']
    field_class = fields.SlugField

class TimeFieldTestCase(BaseFieldTestCase):
    field_class = fields.TimeField

#begin complex field types

class SchemaTypeFieldTestCase(BaseFieldTestCase):
    field_class = fields.SchemaTypeField
    
    def get_field_kwargs(self):
        return {'schemas': dict()}

class SchemaFieldTestCase(BaseFieldTestCase):
    field_class = fields.SchemaField
    EXAMPLE_VALUES = [SimpleSchema(charfield='charmander')]
    EXAMPLE_PRIMITIVE_VALUES = [{'charfield':'charmander'}]
    NULL_VALUE = SimpleSchema()
    
    def get_field_kwargs(self):
        return {'schema': SimpleSchema}

class TypedSchemaFieldTestCase(BaseFieldTestCase):
    field_class = fields.TypedSchemaField
    
    def get_field_kwargs(self):
        return {'schemas': dict()}

class ListFieldTestCase(BaseFieldTestCase):
    field_class = fields.ListField
    EXAMPLE_VALUES = [[], ['a','b','c']]
    EXAMPLE_PRIMITIVE_VALUES = [[], ['a','b','c']]
    NULL_VALUE = []
    
    def get_field_kwargs(self):
        return {'subfield': fields.CharField()}

class SetFieldTestCase(BaseFieldTestCase):
    field_class = fields.SetField
    EXAMPLE_VALUES = [[], ['a','b','c']]
    EXAMPLE_PRIMITIVE_VALUES = [[], ['a','b','c']]
    NULL_VALUE = set()
    
    def get_field_kwargs(self):
        return {'subfield': fields.CharField()}

class DictFieldTestCase(BaseFieldTestCase):
    field_class = fields.DictField
    EXAMPLE_VALUES = [{}, {'a':1}]
    EXAMPLE_PRIMITIVE_VALUES = [{}, {'a':1}]
    NULL_VALUE = dict()
    
    def get_field_kwargs(self):
        return {}

class ReferenceFieldTestCase(BaseFieldTestCase):
    field_class = fields.ReferenceField
    
    def get_field_kwargs(self):
        return {'document': SimpleDocument}

class DocumentSetFieldTestCase(BaseFieldTestCase):
    field_class = fields.DocumentSetField
    NULL_VALUE = set()
    
    def get_field_kwargs(self):
        return {'document': SimpleDocument}

class ModelReferenceFieldTestCase(BaseFieldTestCase):
    field_class = fields.ModelReferenceField
    
    def get_example_values(self):
        return [ContentType.objects.all()[0]]
    
    def get_field_kwargs(self):
        return {'model': ContentType}

class ModelSetFieldTestCase(BaseFieldTestCase):
    field_class = fields.ModelSetField
    NULL_VALUE = set()
    
    def get_field_kwargs(self):
        return {'model': ContentType}




from dockit.forms.widgets import PrimitiveListWidget

from django.conf import settings

class AdminPrimitiveListWidget(PrimitiveListWidget):
    class Media:
        css = {'all': ('%sadmin/css/primitivelist.css' % settings.STATIC_URL,)}
        js = ('%sadmin/js/primitivelist.js' % settings.STATIC_URL,)





from dockit import schema
from dockit.schema.schema import create_document, create_schema
from dockit.forms.forms import document_to_dict, fields_for_document, DocumentForm, documentform_factory

from django.utils import unittest

class SimpleDocument(schema.Document): #TODO make a more complex testcase
    charfield = schema.CharField()

class FormUtilsTestCase(unittest.TestCase):
    def test_document_to_dict(self):
        obj = SimpleDocument(_python_data={'charfield':'charmander'})
        data = document_to_dict(SimpleDocument, obj)
        self.assertTrue('charfield' in data)
        self.assertEqual(data['charfield'], 'charmander')
    
    def test_fields_for_document(self):
        fields = fields_for_document(SimpleDocument)
        self.assertTrue('charfield' in fields)
    
    def test_form_for_schema(self):
        form_cls = documentform_factory(SimpleDocument)

class FormTestCase(unittest.TestCase):
    def setUp(self):
        super(FormTestCase, self).setUp()
        self.form_cls = documentform_factory(SimpleDocument)
    
    def test_blank_init(self):
        form = self.form_cls()
        self.assertFalse(form.is_bound)
    
    def test_data_init(self):
        form = self.form_cls(data={'charfield':'charmander'})
        self.assertTrue(form.is_valid())
    
    def test_instance_init(self):
        obj = SimpleDocument(charfield='charmander')
        form = self.form_cls(instance=obj)
        self.assertFalse(form.is_bound)
    
    def test_instance_update(self):
        obj = SimpleDocument(charfield='charmander')
        form = self.form_cls(instance=obj, data={'charfield':'charmander2'})
        self.assertTrue(form.is_bound)
        self.assertTrue(form.is_valid())
        
        obj2 = form.save()
        self.assertEqual(obj2.charfield, 'charmander2')
    
    def test_instance_create(self):
        form = self.form_cls(data={'charfield':'charmander'})
        self.assertTrue(form.is_valid())
        obj = form.save()
        self.assertEqual(obj.charfield, 'charmander')




from dockit import schema

class SimpleSchema(schema.Schema): #TODO make a more complex testcase
    charfield = schema.CharField()

class SimpleDocument(schema.Document): #TODO make a more complex testcase
    charfield = schema.CharField()
